@@ -4,4 +4,4 @@
 from .continuous_transform import cwt
 from .conv_fwt import wavedec, waverec
 from .conv_fwt_2d import wavedec2, waverec2
-from .packets import WaveletPacket
+from .packets import WaveletPacket, WaveletPacket2D
