@@ -8,49 +8,64 @@
 from typing import List, Optional, Tuple, Union
 
 import jax
+import jax.lax
 import jax.numpy as jnp
 import pywt
 
-from .utils import Wavelet
+from .utils import _as_wavelet
 
 
 def wavedec(
     data: jnp.ndarray,
-    wavelet: Wavelet,
+    wavelet: pywt.Wavelet,
     level: Optional[int] = None,
     mode: str = "reflect",
+    precision: str = "highest",
 ) -> List[jnp.ndarray]:
-    """Compute the one dimensional analysis wavelet transform of the last dimension.
+    """Compute the analysis wavelet transform of the last dimension.
 
     Args:
-        data (jnp.array): Input data array of shape [batch, channels, time]
-        wavelet (Wavelet): The named tuple containing the wavelet filter arrays.
-        level (int): Max scale level to be used, of none as many levels as possible are
+        data (jnp.ndarray): Input data array of shape [batch, time].
+        wavelet (pywt.Wavelet): The named tuple containing the wavelet
+                    filter arrays.
+        level (int): Max scale level to be used,
+                     of none as many levels as possible are
                      used. Defaults to None.
-        mode: The padding used to extend the input signal. Choose reflect or symmetric.
+        mode (str): The padding used to extend the input signal. Choose reflect, symmetric or zero.
             Defaults to reflect.
+        precision (str): The desired precision, choose "fastest", "high" or "highest".
+            Defaults to "highest".
+
 
     Returns:
-        list: List containing the wavelet coefficients.
+        list: List containing the wavelet coefficients of shape [batch_size, coefficients].
             The coefficients are in pywt order:
             [cA_n, cD_n, cD_n-1, …, cD2, cD1].
             A denotes approximation and D detail coefficients.
 
+    Raises:
+        ValueError: If the dimensionality of the input data array is unsupported.
+
     Examples:
         >>> import pywt
         >>> import jaxwt as jwt
-        >>> import jax.numpy as np
+        >>> import jax.numpy as jnp
         >>> # generate an input of even length.
         >>> data = jnp.array([0., 1., 2., 3, 4, 5, 5, 4, 3, 2, 1, 0])
-        >>> jwt.wavedec(data, pywt.Wavelet('haar'),
-                        mode='reflect', level=2)
+        >>> jwt.wavedec(data, wavelet=pywt.Wavelet('haar'), level=2)
     """
+    wavelet = _as_wavelet(wavelet)
     if len(data.shape) == 1:
         # add channel and batch dimension.
         data = jnp.expand_dims(jnp.expand_dims(data, 0), 0)
-    if len(data.shape) == 2:
+    elif len(data.shape) == 2:
         # add the channel dimension.
         data = jnp.expand_dims(data, 1)
+    else:
+        raise ValueError(
+            "Wavedec only supports up to two input dimensions. \
+             Optionally-batched one dimensional inputs work."
+        )
 
     dec_lo, dec_hi, _, _ = _get_filter_arrays(wavelet, flip=True, dtype=data.dtype)
     filt_len = dec_lo.shape[-1]
@@ -71,21 +86,29 @@ def wavedec(
                 2,
             ],
             dimension_numbers=("NCH", "OIH", "NCH"),
+            precision=jax.lax.Precision(precision),
         )
         res_lo, res_hi = jnp.split(res, 2, 1)
-        result_lst.append(res_hi)
-    result_lst.append(res_lo)
+        result_lst.append(res_hi.squeeze(1))
+    result_lst.append(res_lo.squeeze(1))
     result_lst.reverse()
     return result_lst
 
 
-def waverec(coeffs: List[jnp.ndarray], wavelet: Wavelet) -> jnp.ndarray:
+def waverec(
+    coeffs: List[jnp.ndarray],
+    wavelet: pywt.Wavelet,
+    precision: str = "highest",
+) -> jnp.ndarray:
     """Reconstruct the original signal in one dimension.
 
     Args:
         coeffs (list): Wavelet coefficients, typically produced by the wavedec function.
-        wavelet (Wavelet): The named tuple containing the wavelet filters used to evaluate
+            List entries of shape [batch_size, coefficients] work.
+        wavelet (pywt.Wavelet): The named tuple containing the wavelet filters used to evaluate
                               the decomposition.
+        precision (str): The desired precision, choose "fastest", "high" or "highest".
+            Defaults to "highest".
 
     Returns:
         jnp.array: Reconstruction of the original data.
@@ -93,13 +116,13 @@ def waverec(coeffs: List[jnp.ndarray], wavelet: Wavelet) -> jnp.ndarray:
     Examples:
         >>> import pywt
         >>> import jaxwt as jwt
-        >>> import jax.numpy as np
+        >>> import jax.numpy as jnp
         >>> # generate an input of even length.
         >>> data = jnp.array([0., 1., 2., 3, 4, 5, 5, 4, 3, 2, 1, 0])
-        >>> transformed = jwt.wavedec(data, pywt.Wavelet('haar'),
-                          mode='reflect', level=2)
+        >>> transformed = jwt.wavedec(data, pywt.Wavelet('haar'))
         >>> jwt.waverec(transformed, pywt.Wavelet('haar'))
     """
+    wavelet = _as_wavelet(wavelet)
     # lax's transpose conv requires filter flips in contrast to pytorch.
     _, _, rec_lo, rec_hi = _get_filter_arrays(wavelet, flip=True, dtype=coeffs[0].dtype)
     filt_len = rec_lo.shape[-1]
@@ -108,7 +131,7 @@ def waverec(coeffs: List[jnp.ndarray], wavelet: Wavelet) -> jnp.ndarray:
     res_lo = coeffs[0]
     for c_pos, res_hi in enumerate(coeffs[1:]):
         # print('shapes', res_lo.shape, res_hi.shape)
-        res_lo = jnp.concatenate([res_lo, res_hi], 1)
+        res_lo = jnp.stack([res_lo, res_hi], 1)
         res_lo = jax.lax.conv_transpose(
             lhs=res_lo,
             rhs=filt,
@@ -117,8 +140,10 @@ def waverec(coeffs: List[jnp.ndarray], wavelet: Wavelet) -> jnp.ndarray:
                 2,
             ],
             dimension_numbers=("NCH", "OIH", "NCH"),
+            precision=jax.lax.Precision(precision),
         )
         res_lo = _fwt_unpad(res_lo, filt_len, c_pos, coeffs)
+        res_lo = res_lo.squeeze(1)
     return res_lo
 
 
@@ -136,7 +161,6 @@ def _fwt_unpad(
         if nex_len != pred_len:
             padl += 1
             pred_len = res_lo.shape[-1] - padl
-            # assert nex_len == pred_len, 'padding error, please open an issue on github '
     if padl == 0:
         res_lo = res_lo[..., padr:]
     else:
@@ -148,7 +172,7 @@ def _fwt_pad(data: jnp.ndarray, filt_len: int, mode: str = "reflect") -> jnp.nda
     """Pad an input to ensure our fwts are invertible.
 
     Args:
-        data (jnp.array): The input array.
+        data (jnp.ndarray): The input array.
         filt_len (int): The length of the wavelet filters
         mode (str): How to pad. Defaults to "reflect".
 
@@ -182,12 +206,12 @@ def _fwt_pad(data: jnp.ndarray, filt_len: int, mode: str = "reflect") -> jnp.nda
 
 
 def _get_filter_arrays(
-    wavelet: Wavelet, flip: bool, dtype: jnp.dtype = jnp.float64  # type: ignore
+    wavelet: pywt.Wavelet, flip: bool, dtype: jnp.dtype = jnp.float64  # type: ignore
 ) -> Tuple[jnp.ndarray, jnp.ndarray, jnp.ndarray, jnp.ndarray]:
     """Extract the filter coefficients from an input wavelet object.
 
     Args:
-        wavelet (Wavelet): A pywt-style input wavelet.
+        wavelet (pywt.Wavelet): A pywt-style input wavelet.
         flip (bool): If true flip the input coefficients.
         dtype: The desired precision. Defaults to jnp.float64 .
 
@@ -210,6 +234,7 @@ def _get_filter_arrays(
 
     if isinstance(wavelet, str):
         wavelet = pywt.Wavelet(wavelet)
+        dec_lo, dec_hi, rec_lo, rec_hi = wavelet.filter_bank
     elif type(wavelet) is pywt.Wavelet:
         dec_lo, dec_hi, rec_lo, rec_hi = wavelet.filter_bank
     else:
