@@ -8,48 +8,66 @@ from typing import List, Optional, Tuple, Union
 import jax
 import jax.numpy as jnp
 import pywt
-from jax.config import config
 
 from .conv_fwt import _get_filter_arrays
-from .utils import Wavelet
-
-config.update("jax_enable_x64", True)
+from .utils import _as_wavelet
 
 
 def wavedec2(
     data: jnp.ndarray,
-    wavelet: Wavelet,
+    wavelet: pywt.Wavelet,
     level: Optional[int] = None,
     mode: str = "reflect",
+    precision: str = "highest",
 ) -> List[Union[jnp.ndarray, Tuple[jnp.ndarray, jnp.ndarray, jnp.ndarray]]]:
     """Compute the two dimensional wavelet analysis transform on the last two dimensions of the input data array.
 
     Args:
-        data (jnp.array): Jax array containing the data to be transformed. Assumed shape:
+        data (jnp.ndarray): Jax array containing the data to be transformed. Assumed shape:
                          [batch size, hight, width].
-        wavelet (Wavelet): A namedtouple containing the filters for the transformation.
+        wavelet (pywt.Wavelet): A namedtouple containing the filters for the transformation.
         level (int): The max level to be used, if not set as many levels as possible
                                will be used. Defaults to None.
-        mode (str): The desired padding mode. Choose reflect or symmetric.
+        mode (str): The desired padding mode. Choose reflect, symmetric or zero.
             Defaults to reflect.
+        precision (str): The desired precision, choose "fastest", "high" or "highest".
+            Defaults to "highest".
 
     Returns:
-        list: The wavelet coefficients in a nested list.
+        list: The wavelet coefficients of shape [batch, height, width] in a nested list.
             The coefficients are in pywt order. That is:
             [cAn, (cHn, cVn, cDn), … (cH1, cV1, cD1)].
             A denotes approximation, H horizontal, V vertical
             and D diagonal coefficients.
 
+    Raises:
+        ValueError: If the dimensionality of the input data array is unsupported.
+
     Examples:
-        >>> import pywt, scipy.misc
+        >>> import pywt, scipy.datasets
         >>> import jaxwt as jwt
         >>> import jax.numpy as jnp
-        >>> face = jnp.transpose(scipy.misc.face(), [2, 0, 1]).astype(jnp.float64)
-        >>> jwt.wavedec2(face, pywt.Wavelet("haar"), level=2, mode="reflect")
+        >>> face = jnp.transpose(scipy.datasets.face(), [2, 0, 1])
+        >>> face = face.astype(jnp.float64)
+        >>> jwt.wavedec2(face, pywt.Wavelet("haar"), level=2)
     """
-    data = jnp.expand_dims(data, 1)
-    dec_lo, dec_hi, _, _ = _get_filter_arrays(wavelet, flip=True)
-    dec_filt = construct_2d_filt(lo=dec_lo, hi=dec_hi)
+    wavelet = _as_wavelet(wavelet)
+
+    if len(data.shape) == 2:
+        data = jnp.expand_dims(data, (0, 1))
+    elif len(data.shape) == 3:
+        data = jnp.expand_dims(data, 1)
+    elif len(data.shape) == 4:
+        raise ValueError(
+            "Wavedec2 does not support four input dimensions. \
+             Optionally-batched two dimensional inputs work."
+        )
+    elif len(data.shape) == 1:
+        raise ValueError("Wavedec2 needs more than one input dimension to work.")
+
+    # data = jnp.expand_dims(data, 1)
+    dec_lo, dec_hi, _, _ = _get_filter_arrays(wavelet, flip=True, dtype=data.dtype)
+    dec_filt = _construct_2d_filt(lo=dec_lo, hi=dec_hi)
 
     if mode == "zero":
         # translate pywt to numpy.
@@ -72,6 +90,7 @@ def wavedec2(
             padding="VALID",
             window_strides=[2, 2],
             dimension_numbers=("NCHW", "OIHW", "NCHW"),
+            precision=jax.lax.Precision(precision),
         )
         res_ll, res_lh, res_hl, res_hh = jnp.split(res, 4, 1)
         result_lst.append((res_lh.squeeze(1), res_hl.squeeze(1), res_hh.squeeze(1)))
@@ -82,7 +101,8 @@ def wavedec2(
 
 def waverec2(
     coeffs: List[Union[jnp.ndarray, Tuple[jnp.ndarray, jnp.ndarray, jnp.ndarray]]],
-    wavelet: Wavelet,
+    wavelet: pywt.Wavelet,
+    precision: str = "highest",
 ) -> jnp.ndarray:
     """Compute a two dimensional synthesis wavelet transfrom.
 
@@ -90,7 +110,9 @@ def waverec2(
 
     Args:
         coeffs (list): The input coefficients, typically the output of wavedec2.
-        wavelet (Wavelet): The named tuple contining the filters used to compute the analysis transform.
+        wavelet (pywt.Wavelet): The named tuple contining the filters used to compute the analysis transform.
+        precision (str): The desired precision, choose "fastest", "high" or "highest".
+            Defaults to "highest".
 
     Returns:
         jnp.array: Reconstruction of the original input data array of shape [batch, height, width].
@@ -99,22 +121,24 @@ def waverec2(
         ValueError: If `coeffs` is not in the shape as it is returned from `wavedec2`.
 
     Example:
-        >>> import pywt, scipy.misc
+        >>> import pywt, scipy.datasets
         >>> import jaxwt as jwt
         >>> import jax.numpy as jnp
-        >>> face = jnp.transpose(scipy.misc.face(), [2, 0, 1]).astype(jnp.float64)
-        >>> transformed = jwt.wavedec2(face, pywt.Wavelet("haar"), level=2, mode="reflect")
+        >>> face = jnp.transpose(scipy.datasets.face(), [2, 0, 1])
+        >>> face = face.astype(jnp.float64)
+        >>> transformed = jwt.wavedec2(face, pywt.Wavelet("haar"))
         >>> jwt.waverec2(transformed, pywt.Wavelet("haar"))
 
 
     """
+    wavelet = _as_wavelet(wavelet)
     if not isinstance(coeffs[0], jnp.ndarray):
         raise ValueError(
             "First element of coeffs must be the approximation coefficient tensor."
         )
     _, _, rec_lo, rec_hi = _get_filter_arrays(wavelet, flip=True, dtype=coeffs[0].dtype)
     filt_len = rec_lo.shape[-1]
-    rec_filt = construct_2d_filt(lo=rec_lo, hi=rec_hi)
+    rec_filt = _construct_2d_filt(lo=rec_lo, hi=rec_hi)
     rec_filt = jnp.transpose(rec_filt, [1, 0, 2, 3])
 
     res_ll = jnp.expand_dims(coeffs[0], 1)
@@ -134,6 +158,7 @@ def waverec2(
             padding="VALID",
             strides=[2, 2],
             dimension_numbers=("NCHW", "OIHW", "NCHW"),
+            precision=jax.lax.Precision(precision),
         )
         # remove the padding
         padl = (2 * filt_len - 3) // 2
@@ -169,12 +194,12 @@ def waverec2(
     return res_ll.squeeze(1)
 
 
-def construct_2d_filt(lo: jnp.ndarray, hi: jnp.ndarray) -> jnp.ndarray:
+def _construct_2d_filt(lo: jnp.ndarray, hi: jnp.ndarray) -> jnp.ndarray:
     """Construct 2d filters from 1d inputs using outer products.
 
     Args:
-        lo (jnp.array): 1d lowpass input filter of size [1, length].
-        hi (jnp.array): 1d highpass input filter of size [1, length].
+        lo (jnp.ndarray): 1d lowpass input filter of size [1, length].
+        hi (jnp.ndarray): 1d highpass input filter of size [1, length].
 
     Returns
         jnp.array: 2d filter arrays of shape [4, 1, length, length].
